@@ -200,6 +200,8 @@ class HeapRegion: public G1ContiguousSpace {
   // issues.)
   HeapRegionRemSet* _rem_set;
 
+  uint _processing_order;
+
   // Auxiliary functions for scan_and_forward support.
   // See comments for CompactibleSpace for more information.
   inline HeapWord* scan_limit() const {
@@ -463,6 +465,14 @@ class HeapRegion: public G1ContiguousSpace {
     return _rem_set;
   }
 
+  uint processing_order() {
+    return _processing_order;
+  }
+
+  void set_processing_order(uint processing_order) {
+    _processing_order = processing_order;
+  }
+
   inline bool in_collection_set() const;
 
   // Methods used by the HeapRegionSetBase class and subclasses.
