@@ -270,7 +270,7 @@ void StringDedupTable::transfer(StringDedupEntry** pentry, StringDedupTable* des
 }
 
 bool StringDedupTable::equals(typeArrayOop value1, typeArrayOop value2) {
-  return (oopDesc::equals(value1, value2) ||
+  return (value1 == value2 ||
           (value1->length() == value2->length() &&
           (!memcmp(value1->base(T_BYTE),
                     value2->base(T_BYTE),
