@@ -52,13 +52,8 @@ class LatestMethodCache : public CHeapObj<mtClass> {
   Klass*                _klass;
   int                   _method_idnum;
 
-  static bool _is_redefining_gc_run;
-
  public:
 
-   static bool is_redefining_gc_run()               { return _is_redefining_gc_run; }
-   static void set_redefining_gc_run(bool b)        { _is_redefining_gc_run = b;    }
-
   LatestMethodCache()   { _klass = NULL; _method_idnum = -1; }
   ~LatestMethodCache()  { _klass = NULL; _method_idnum = -1; }
 
@@ -354,6 +349,9 @@ class Universe: AllStatic {
   // Function to initialize these
   static void initialize_known_methods(TRAPS);
 
+  // Enhanced class redefinition
+  static void reinitialize_loader_addClass_method(TRAPS);
+
   static oop          null_ptr_exception_instance()   { return _null_ptr_exception_instance;   }
   static oop          arithmetic_exception_instance() { return _arithmetic_exception_instance; }
   static oop          virtual_machine_error_instance() { return _virtual_machine_error_instance; }
