@@ -448,6 +448,8 @@ void ConstantPoolCacheEntry::set_method_handle_common(const constantPoolHandle&
     resolved_references->obj_at_put(method_type_index, method_type());
   }
 
+  OrderAccess::release_store(&_flags, _flags & ~(1u << is_f1_null_dcevm_shift));
+
   release_set_f1(adapter());  // This must be the last one to set (see NOTE above)!
 
   // The interpreter assembly code does not check byte_2,
@@ -654,16 +656,8 @@ void ConstantPoolCacheEntry::clear_entry() {
   _indices = constant_pool_index();
 
   if (clearData) {
-    if (!is_resolved_reference()) {
-      _f2 = 0;
-    }
-    // FIXME: (DCEVM) we want to clear flags, but parameter size is actually used
-    // after we return from the method, before entry is re-initialized. So let's
-    // keep parameter size the same.
-    // For example, it's used in TemplateInterpreterGenerator::generate_return_entry_for
-    // Also, we need to keep flag marking entry as one containing resolved_reference
-    _flags &= parameter_size_mask | (1 << is_resolved_ref_shift);
-    _f1 = NULL;
+     // DCEVM: do not clear f1 now, since it can be used before cache entry is re-resolved
+    _flags |= (1 << is_f1_null_dcevm_shift);
   }
 }
 
