@@ -54,7 +54,9 @@
 #include "utilities/events.hpp"
 #include "oops/constantPool.inline.hpp"
 #include "gc/cms/cmsHeap.hpp"
+#if INCLUDE_G1GC
 #include "gc/g1/g1CollectedHeap.hpp"
+#endif
 #include "gc/shared/dcevmSharedGC.hpp"
 
 Array<Method*>* VM_EnhancedRedefineClasses::_old_methods = NULL;
@@ -536,36 +538,47 @@ void VM_EnhancedRedefineClasses::doit() {
     // mark such nmethod's as "scavengable".
     // For now, mark all nmethod's as scavengable that are not scavengable already
     if (ScavengeRootsInCode) {
+#if INCLUDE_G1GC
       if (UseG1GC) {
         // G1 holds references to nmethods in regions based on oops values. Since oops in nmethod can be changed in ChangePointers* closures
         // we unregister nmethods from G1 heap, then closures are processed (oops are changed) and finally we register nmethod to G1 again
         CodeCache::nmethods_do(unregister_nmethod_g1);
       } else {
+#endif
         CodeCache::nmethods_do(mark_as_scavengable);
+#if INCLUDE_G1GC
       }
+#endif
     }
 
     Universe::heap()->ensure_parsability(false);
+#if INCLUDE_G1GC
     if (UseG1GC) {
       if (log_is_enabled(Info, redefine, class, timer)) {
         _timer_heap_iterate.start();
       }
+      // returns after the iteration is finished
       G1CollectedHeap::heap()->object_par_iterate(&objectClosure);
       _timer_heap_iterate.stop();
     } else {
+#endif
       if (log_is_enabled(Info, redefine, class, timer)) {
         _timer_heap_iterate.start();
       }
       Universe::heap()->object_iterate(&objectClosure);
       _timer_heap_iterate.stop();
+#if INCLUDE_G1GC
     }
+#endif
 
     Universe::root_oops_do(&oopClosureNoBarrier);
 
+#if INCLUDE_G1GC
     if (UseG1GC) {
       // this should work also for other GCs
       CodeCache::nmethods_do(register_nmethod_g1);
     }
+#endif
 
   }
   log_trace(redefine, class, obsolete, metadata)("After updating instances");
@@ -647,12 +660,8 @@ void VM_EnhancedRedefineClasses::doit() {
   // TODO: explain...
   ciObjectFactory::resort_shared_ci_metadata();
 
-  // FIXME - check if it was in JDK8. Copied from standard JDK9 hotswap.
-  //MethodDataCleaner clean_weak_method_links;
-  //ClassLoaderDataGraph::classes_do(&clean_weak_method_links);
-
-  // Disable any dependent concurrent compilations
-  SystemDictionary::notice_modification();
+  // notice_modification() was removed in 11.0.14
+  // SystemDictionary::notice_modification();
 
   // Set flag indicating that some invariants are no longer true.
   // See jvmtiExport.hpp for detailed explanation.
@@ -1017,7 +1026,7 @@ int VM_EnhancedRedefineClasses::calculate_redefinition_flags(InstanceKlass* new_
 
     cur_klass = new_class->super();
     while (cur_klass != NULL) {
-      if (!the_class->is_subclass_of(cur_klass->old_version())) {
+      if (!the_class->is_subclass_of(cur_klass->is_redefining() ? cur_klass->old_version() : cur_klass)) {
         log_info(redefine, class, load)("added super class %s", cur_klass->name()->as_C_string());
         result = result | Klass::ModifyClass | Klass::ModifyInstances;
       }
@@ -2061,12 +2070,23 @@ void VM_EnhancedRedefineClasses::dump_methods() {
 class AffectedKlassClosure : public KlassClosure {
  private:
    GrowableArray<Klass*>* _affected_klasses;
+   bool _is_anonymous;
  public:
-  AffectedKlassClosure(GrowableArray<Klass*>* affected_klasses) : _affected_klasses(affected_klasses) {}
+  AffectedKlassClosure(GrowableArray<Klass*>* affected_klasses) : _affected_klasses(affected_klasses), _is_anonymous(false) {}
+
+  bool is_anonymous() { return _is_anonymous; }
+  void set_anonymous(bool value) { _is_anonymous = value; }
 
   void do_klass(Klass* klass) {
     assert(!_affected_klasses->contains(klass), "must not occur more than once!");
 
+    if (_is_anonymous && klass->is_instance_klass()) {
+      InstanceKlass *ik = InstanceKlass::cast(klass);
+      if (ik->is_not_initialized()) {
+        return;  // anonymous class does not need to be initialized
+      }
+    }
+
     if (klass->new_version() != NULL) {
       return;
     }
@@ -2125,11 +2145,13 @@ jvmtiError VM_EnhancedRedefineClasses::find_sorted_affected_classes(TRAPS) {
   // ClassLoaderDataGraph::classes_do(&closure);
 
   // 1. Scan over dictionaries
+  closure.set_anonymous(false);
   ClassLoaderDataGraph::dictionary_classes_do(&closure);
 
   // 2. Anonymous class is not in dictionary, we have to iterate anonymous cld directly, but there is race cond...
   // TODO: review ... anonymous class is added to cld before InstanceKlass initialization,
   //                  find out how to check if the InstanceKlass is initialized
+  closure.set_anonymous(true);
   ClassLoaderDataGraph::anonymous_classes_do(&closure);
 
   log_trace(redefine, class, load)("%d classes affected", _affected_klasses->length());
