@@ -420,7 +420,7 @@ oop Reflection::array_component_type(oop mirror, TRAPS) {
     assert(lower_dim->is_array_klass(), "just checking");
     result2 = lower_dim->java_mirror();
   }
-  assert(oopDesc::equals(result, result2), "results must be consistent");
+  assert(result == result2, "results must be consistent");
 #endif //ASSERT
   return result;
 }
@@ -660,7 +660,7 @@ bool Reflection::verify_member_access(const Klass* current_class,
                                       TRAPS) {
 
   // (DCEVM) Decide accessibility based on active version
-  if (current_class != NULL) {
+  if (AllowEnhancedClassRedefinition && current_class != NULL) {
     current_class = current_class->active_version();
   }
 
